
#include <mpi.h>
extern "C"
{
#include <stdio.h>
#include <unistd.h>
#include <getopt.h>
}

#include <vtkAppendFilter.h>
#include <vtkSmartPointer.h>
#include <vtkSphereSource.h>
#include <vtkUnstructuredGrid.h>

#include <catalyzer.h>

vtkSmartPointer<vtkUnstructuredGrid> getUG(double startTheta, double endTheta)
{
    /* Generate a sphere portion */
    vtkSmartPointer<vtkSphereSource> sphereSource = vtkSmartPointer<vtkSphereSource>::New();
        sphereSource->SetRadius(32.0);
        sphereSource->SetStartTheta(startTheta);
        sphereSource->SetEndTheta(endTheta);
        sphereSource->Update();

    /* COnvert PolyData info UnstructuredGrid */
    vtkSmartPointer<vtkAppendFilter> appendFilter = vtkSmartPointer<vtkAppendFilter>::New();
#if VTK_MAJOR_VERSION <= 5
        appendFilter->AddInput(sphereSource->GetOutput());
#else
        appendFilter->AddInputData(sphereSource->GetOutput());
#endif
        appendFilter->Update();

    /* Copy the output UnstructuredGrid */
    vtkSmartPointer<vtkUnstructuredGrid> unstructuredGrid = vtkSmartPointer<vtkUnstructuredGrid>::New();
    unstructuredGrid->ShallowCopy(appendFilter->GetOutput());

    return unstructuredGrid;
}

int main(int argc, char ** argv)
{
    int enableSteering = 0;
    std::string hostname = "localhost";
    int port = 22222;

    /* Parsable options */
    static struct option long_options[] = {
        {"enable-steering", no_argument, 0, 's' },
        {"hostname",  required_argument, 0, 'h' },
        {"port",      required_argument, 0, 'p' },
        {0,           0,                 0,  0 }
    };

    char c;
    while ((c = getopt_long(argc, argv, "sh:p:", long_options, NULL)) != -1)
    {
        //check to see if a single character or long option came through
        switch (c)
        {
            case 's':
                enableSteering = 1;
                break;
            case 'h':
                hostname = optarg;
                break;
            case 'p':
                port = strtol(optarg, NULL, 10); // or copy it if you want to
                break;
        }
    }

    //Initialize the MPI environment
    MPI_Init(NULL, NULL);

    // Get the number of processes
    int worldSize;
    MPI_Comm_size(MPI_COMM_WORLD, &worldSize);

    // Get the rank of the process
    int worldRank;
    MPI_Comm_rank(MPI_COMM_WORLD, &worldRank);
    
    // Print off a hello world message
    printf("[%d:%d] Steering=%d ; InsituHostname=%s ; InsituPort=%d\n", worldRank, worldSize, enableSteering, hostname.c_str(), port);

    Catalyzer * catalyzer = new Catalyzer(MPI_COMM_WORLD);
    if(enableSteering)
    { catalyzer->EnableSteering(); }
    catalyzer->SetInsituHostname(hostname.c_str());
    catalyzer->SetInsituPort(port);

    double d = 1.0;
    int i = 0;
    int valid = 0;
    int time = 0.0;

    /* compute the portion of sphere each processor will generate (in degrees) */
    double deltaTheta = 360.0 / ((double)(worldSize));
    double startTheta = (double)(worldRank * deltaTheta);
    double endTheta = (double)(worldRank * deltaTheta + deltaTheta) - 1.0;
    printf("[%d:%d] StartTheta=%f ; EndTheta=%f ; deltaTheta=%f\n", worldRank, worldSize, startTheta, endTheta, deltaTheta);
    for(; i < 360; i++, time += 1.0)
    {
        if(enableSteering)
        {
            catalyzer->GetSteeringValue(&d, &valid);
            printf("SteeringValue %f %d\n", d, valid);
        }
        startTheta = (double)((int)(startTheta + d) % 360);
        endTheta = (double)((int)(endTheta + d) % 360);
        vtkSmartPointer<vtkUnstructuredGrid> ug = getUG(startTheta, endTheta);
        catalyzer->Save(i, time, ug);
    }

    // Free memory
    delete catalyzer;

    // Finalize the MPI environment.
    MPI_Finalize();

    return 0;
}
